@@ -2,18 +2,22 @@
 
 from __future__ import absolute_import
 import datetime
+import sys
 
 from pymycobot.generate import MyCobotCommandGenerator
 from pymycobot.Interface import MyBuddyCommandGenerator
 from pymycobot.mycobot import MyCobot
 from pymycobot.mybuddy import MyBuddy
+from pymycobot.mecharm import MechArm
 from pymycobot.mypalletizer import MyPalletizer
 from pymycobot.mycobotsocket import MyCobotSocket
 from pymycobot.genre import Angle, Coord
 from pymycobot import utils
 from pymycobot.mybuddysocket import MyBuddySocket
-from pymycobot.mira import Mira
+from pymycobot.ultraArm import ultraArm
 from pymycobot.mybuddybluetooth import MyBuddyBlueTooth
+from pymycobot.mypalletizersocket import MyPalletizerSocket
+    
 
 __all__ = [
     "MyCobot",
@@ -27,10 +31,17 @@ __all__ = [
     "MyBuddy",
     "MyBuddySocket",
     "MyBuddyBlueTooth",
-    "Mira"
+    "ultraArm",
+    "MyPalletizerSocket",
+    "MechArm"
 ]
 
-__version__ = "2.9.6"
+
+if sys.platform == "linux":
+    from pymycobot.mybuddyemoticon import MyBuddyEmoticon
+    __all__.append("MyBuddyEmoticon")
+
+__version__ = "2.9.8"
 __author__ = "Elephantrobotics"
 __email__ = "weiquan.xu@elephantrobotics.com"
 __git_url__ = "https://github.com/elephantrobotics/pymycobot"
